function create_tables()
% Even when the simulations are not finished, result.mat is saved and one can check the result by running this function
[table1ds, table1var] = create_table1('table1dat.mat');
[table2ds, table2var] = create_table1('table2dat.mat');
save tables.mat
end

function [table1ds, table1var] = create_table1(matfile)
if ~exist(matfile,'file')
    table1ds = 0;
    table1var = 0;
    return;
end
load(matfile)

n_rows = length(list_DGP)*length(list_n);
table1ds = dataset({cell(n_rows,1),'DGP'},{NaN(n_rows,3),'theta1', 'theta2', 'n'});
n_pre_var = size(table1ds,2);

varnames = cell(1,length(list_method));
for mi=1:length(list_method)
    varnames{mi} = list_method{mi}.name;
end
table1ds = [table1ds, dataset({NaN(n_rows,length(varnames)),varnames{:}})];
table1var = table1ds.Properties.VarNames;

ct = 0;
for di=1:length(list_DGP)
    DGP = list_DGP{di};
    for ri=1:length(list_n)
        ct = ct + 1;
        table1ds.DGP(ct) = {get_DGP_sting(DGP)};
        table1ds.theta1(ct) = DGP.theta(1);
        table1ds.theta2(ct) = DGP.theta(2);
        table1ds.n(ct) = list_n(ri);
        for mi=1:length(list_method)
            table1ds.(n_pre_var+mi)(ct) = mean(pass_pr(di,ri,mi,:));
        end
    end
end

end

function name_str = get_DGP_sting(DGP)
name_str = [DGP.proc];
if isfield(DGP,'p')
    name_str = [name_str,', p=',num2str(DGP.p,'%.2f')];
end
if isfield(DGP,'rho')
    name_str = [name_str,', rho=',num2str(DGP.rho,'%.2f')];
end
end
