function main()
data_distribution_only = 0;  % if positive, draw figures only.
S = 1000;  % Number of Monte Carlo we need for this simulation
nboot = 200;  % Number of bootstrapping replications

nworkers = 4;  % number of workers for parallel computing
try
    if all(matlabpool('size')~=[0,nworkers])
        matlabpool close
    end
    matlabpool(nworkers)  % set the number of workers for parallel computing
end

alph = 0.05;  % 1-alph is the confidence level
As = {
    0;
    1;
    };

% list of simulation settings
list_DGP_table1 = cell(0);  % DGP settings
% d.theta: theta values
% d.proc: data generating process
% d.p, d.rho: parameters for DGP
i=0;
i=i+1; d.theta=[.4,.6]; d.proc='IID'; d.p=.5; list_DGP_table1{i}=d; clear d;
i=i+1; d.theta=[.4,.6]; d.proc='IID'; d.p=1; list_DGP_table1{i}=d; clear d;
i=i+1; d.theta=[.4,.6]; d.proc='NonIID'; list_DGP_table1{i}=d; clear d;
i=i+1; d.theta=[.49,.51]; d.proc='IID'; d.p=.5; list_DGP_table1{i}=d; clear d;
i=i+1; d.theta=[.49,.51]; d.proc='IID'; d.p=1; list_DGP_table1{i}=d; clear d;
i=i+1; d.theta=[.49,.51]; d.proc='NonIID'; list_DGP_table1{i}=d; clear d;

if data_distribution_only>0  % draw the graph only
    draw_distribution(S, list_DGP_table1, 256^2);
    return;
end

list_n_table1 = [10^2, 16^2, 20^2, 100^2, 256^2];  

list_method_table1 = cell(0);  % inference methods
% m.name: name of the method
% m.hn: tuning parameter
% m.std: standardize if 1
% m.boot: # of bootstrap replications
i=0;
i=i+1; m.name='EKS';                                                list_method_table1{i}=m; clear m;
i=i+1; m.name='GH1lnln'; m.hn='log(log(n))'; m.std=1; m.boot=nboot; list_method_table1{i}=m; clear m;
i=i+1; m.name='AB';                                   m.boot=nboot; list_method_table1{i}=m; clear m;

% settings for volume
list_DGP_table2 = list_DGP_table1;
list_n_table2 = list_n_table1;
list_method_table2 = list_method_table1;
[g1, g2] = meshgrid(0:.025:1);
% [g1, g2] = meshgrid(0:.5:1);  % for test only
all_thetas = [g1(:) g2(:)];  % all theta values to test

% run
disp('===========================')
disp('table 1: size')
disp('===========================')
simulation_pass_pr(S, alph, As, list_DGP_table1, list_n_table1, list_method_table1, 'true_theta', 'table1dat.mat');
disp('===========================')
disp('table 2: volume')
disp('===========================')
simulation_pass_pr(S, alph, As, list_DGP_table2, list_n_table2, list_method_table2, all_thetas, 'table2dat.mat');

create_tables;  % create the final table
load tables.mat
end

function draw_distribution(S, list_DGP, n)
for di=1:length(list_DGP)
    DGP = list_DGP{di};
    theta = DGP.theta;
    dat = generate_data(theta,S,n,DGP);  % generate data
    freq = mean(dat);
    figure;
    hist(freq,0.35:0.001:0.65);
%     hist(freq,0:0.001:1);
end
end

function simulation_pass_pr(S, alph, As, list_DGP, list_n, list_method, thetas_to_check, matfile)
% computes probabilities of a given theta passes each test
% computes under all combinations of (list_DGP, list_r, list_method)
% S: Number of Monte Carlo simulations
% alph: 1-alph is the confidence level
% As: events to consider
% list_DGP, list_r, list_method: cell arrays
% theta_type: 'true_theta' or 'all'. The result is the size if 'true_theta', and it is the volume of the confidence set if 'all'
% the result will be saved in matfile.m

% initialize variables
switch class(thetas_to_check)
    case 'char'
        if strcmp(thetas_to_check,'true_theta')
            theta_type = thetas_to_check;
            n_theta2 = 1;
        else
            error('ERROR: thetas_to_check is of an unexpected type')
        end
    case 'double'
        theta_type = 'given';
        theta2 = thetas_to_check;
        n_theta2 = size(theta2,1);
    otherwise
        error('ERROR: thetas_to_check is of an unexpected class')
end

pass_pr = NaN(length(list_DGP), length(list_n), length(list_method), n_theta2);  % coverage probabilities
sim_count = 0;  % counter for simulation settings
sim_total = length(list_DGP)*length(list_n)*length(list_method);  % total number of simulation settings
comp_total = length(list_DGP)*sum(list_n)*length(list_method);  % For computing ETA. Complexity is assumed to proportional to n
comp_done = 0;  % For computing ETA
tic;
disp('simulation started')
for di=1:length(list_DGP)
    DGP = list_DGP{di};
    theta = DGP.theta;
    DGP = rmfield(DGP,'theta');  % DGP.theta will not be used. Just to avoid mistakes
    for ni=1:length(list_n)
        n = list_n(ni);

        dat = generate_data(theta,S,n,DGP);  % generate data
        if strcmp(theta_type, 'true_theta')
            theta2 = theta;
        end

% compute the coverage probability
        for mi=1:length(list_method)
            sim_count = sim_count + 1;
            method = list_method{mi};
            switch method.name(1:2)  % check the first two letters
                case 'EK'  % new inference by EKS
                    pass_pr(di,ni,mi,:) =  check_theta_in_CI_EKS(dat,As,theta2,alph);
                case 'GH'  % inference by GH
                    hn = eval(method.hn);
                    pass_pr(di,ni,mi,:) = check_theta_in_CI_GH(dat,As,theta2,alph,method.boot,hn,method.std);
                case 'AB'  % inference by AB
                    pass_pr(di,ni,mi,:) = check_theta_in_CI_AB(dat,As,theta2,alph,method.boot);
                otherwise
                    error('ERROR: method undefined')
            end

            % compute ETA
            comp_done = comp_done + n;
            eta = toc*(comp_total-comp_done)/comp_done;
            disp([num2str(sim_count) '/' num2str(sim_total) ' done. ' ...
                'ETA ' num2str(eta,'%.1f') ' seconds (' num2str(eta/3600,'%.1f') ' hours)'])
        end
    end
    save(matfile, ...   % save results even in the middle of simulations
        'S', 'alph', 'As', ...
        'list_DGP', 'list_n', 'list_method', ...
        'pass_pr', 'thetas_to_check')
    disp([matfile ' saved'])
end
end

function prop_pass = check_theta_in_CI_AB(dat,As,thetas,alph,B)
% checks if theta is in the confidence interval
% dat: n by S (n: sample size, S: # of simulations)
% As: cell array of events - each event is a vector
% thetas: value of parameter to check (each theta is a row), n_thetas by 2
% alph: 1-alpha is the confidence level
% B: # of bootstrap replications
% prop_pass: proportion of simulations where theta is contained in the confidence set, n_thetas by 1

S = size(dat,2);
J = length(As);
n_thetas = size(thetas,1);
prop_pass = NaN(n_thetas,1);

ub = fzero(@(x) (1-chi2cdf(x,J) + 1-chi2cdf(x,J-1))/2-0.05,1);  % upper bound of the critical value (Rosen, 2008)
parfor thi=1:n_thetas
    theta = thetas(thi,:);
    nu_cA = zeros(J,S);  % theoretical min prob (or belief function)
    for j=1:J
        nu_cA(j,:) = get_nu_conj(theta,As{j});
    end
    
    Tn = get_statistic_AB(dat,As,nu_cA);  % statistic
    if all(Tn>ub)
        prop_pass(thi) = 0;
    else
        c = get_critical_value_AB(dat,As,nu_cA,alph,B);  % critical value
%         prop_pass(thi) = mean(Tn <= c);
        prop_pass(thi) = 1 - mean(Tn > c);
    end
end
end

function Tn=get_statistic_AB(dat,As,nu_cA) %%%
% compute AB's statistic
% dat      :  n by S (n :sample size, S: # of simulations)
% As       :  cell array of events- each event is a vector
% nu_cA    :  Upper probability

warning('off','MATLAB:singularMatrix')
warning('off','MATLAB:nearlySingularMatrix')

J=length(As);
n=size(dat,1);
S=size(dat,2);
Tn=zeros(S,1);

Q=get_frequencies(dat,[0,1]); % empirical frequencies of outcomes
P_A=get_freq_events(As,Q); %empirical frequencies of events
m_nbar=nu_cA-P_A; %%% This is J by S dimension

%%% get covariance and correlation matrices (stored as J^2-by-S array) %%%
[sigma, Sigma_nhat, Omega_nhat, ~] = get_correl_AB(P_A,As);
epsilon=0.012; % this value is based on the AB EMA(2012)

for s=1:S
    %%% obtain corrected covariance estimate %%%
    omega = reshape(Omega_nhat(:,s),[J,J]);
    correct_comp=[epsilon-det(omega),0];    % based on p.2808 in AB
    sigma_ntilde=reshape(Sigma_nhat(:,s),[J,J])+max(correct_comp)*diag(sigma(:,s).^2);
    
%%% Here I need to put in if statements for different J we have.    
    %%% solve QP %%%
    settings.verbose=0;
    params.Q=inv(sigma_ntilde);
    params.m=sqrt(n)*m_nbar(:,s);
    
    if J==1; %%% Here just a quick check we use J or m_nbar?
    [~,status] = csolve1(params,settings);
    Tn(s,1)=status.optval;
    
    elseif J==2;
    [~,status] = csolve2(params,settings);
    Tn(s,1)=status.optval;
    
    elseif J==3;
    [~,status] = csolve3(params,settings);
    Tn(s,1)=status.optval;
    
    elseif J==4;
    [~,status] = csolve4(params,settings);
    Tn(s,1)=status.optval;
    
    elseif J==5;
    [~,status] = csolve5(params,settings);
    Tn(s,1)=status.optval;
    
    elseif J==6;
    [~,status] = csolve6(params,settings);
    Tn(s,1)=status.optval;
    end
end
end

function c=get_critical_value_AB(dat,As,nu_cA,alph,B)
% compute AB's critical value
% dat   : n by S (n sample size, S: # of simulation)
% As    : cell array of events - each event is a vector
% nu_cA : Upper probability
% alph  : 1-alpha is the confidence level
% B     : # of simulation
%%% Noice here we do not need tunning parameter since we are going to
%%% choose it based on the get_TP_SCF_AB

J=length(As);
n=size(dat,1);
S=size(dat,2);

Q=get_frequencies(dat,[0,1]);
P_A=get_freq_events(As,Q);
m_nbar=nu_cA-P_A;
[sigma, ~, Omega_nhat, locd] = get_correl_AB(P_A,As);

%%% get the minimum off-diagonal element of correlation matrix %%%
offdiag_Omega=Omega_nhat;
offdiag_Omega(locd,:)=[];
delta=min(offdiag_Omega);

%%% select moment selection parameter and size correction factor %%%
[kappa_hat, eta1_hat, eta2_hat]=get_TP_SCF_AB(delta,J); % get tuning parameters from Table 1 in AB.
Eta_hat=eta1_hat+eta2_hat; %%% size_correction factor

%%% moment selection %%%
momselect_mbar=(n^(1/2))*(m_nbar)./sigma; %sigma is sqrt of diagonal elements.
Kappa=repmat(kappa_hat',J,1);
select_result=momselect_mbar<=Kappa;
moment_selectmatrix=double(select_result); 

%%% step 8 bootstrap to get statistic
rng(12345)
Tn_phat = bootstrp(B,@(x)get_submoment_AB(x,As,nu_cA,m_nbar,moment_selectmatrix),dat);  % bootstrap
c_phat=quantile(Tn_phat,1-alph,1);
c=c_phat'+Eta_hat;
end

function [sigma, Sigma_nhat, Omega_nhat, locd] = get_correl_AB(P_A,As) %%% we need to revise this again
% compute standard devistions, covariance, correlation of moments
% P_A       : J-by S empirical frequeicies of events
% sigma     : standard deviation 
% Sigma_nhat: J^2-by-S covariance
% Omega_nhat: J^2-by-S correlation
% locd      : indices for diagonal elements of Omega_nhat

temp = cellfun(@(x) x(1), As);
ind  = cellfun(@length,As)==1;
outcome = temp(ind)';
q = length(outcome);
J=size(P_A,1);
S = size(P_A,2);
Sigma1=zeros(J^2,S); % 2nd moments of indicators

%%% step 1 assign diagonal elements to Sigma1.
locd = diag(reshape(1:J^2,[J,J])); %%% indices for diagonal elements
Sigma1(locd,:)=P_A;

%%% step 2 assign corresponding off diagonal elements
locoff = zeros(J,J);
for i=1:J
    for j=i+1:J
        for k=1:q
            if intersect(As{i},As{j}) == outcome(k);
                locoff(i,j) = k;
            end
        end
    end
end
locoff = locoff' + locoff;

loc = cell(q,1);
for k=1:q
loc{k} = find(locoff==k);
Sigma1([loc{k}],:)=repmat(P_A(k,:),length(loc{k}),1);
end


%%% step 3 calculate Sigma_nhat
Sigma2 = repmat(P_A,J,1).*kron(P_A,ones(J,1)); % second term of sigma_nhat.
Sigma_nhat=Sigma1-Sigma2;

%%% step 4 calculate Omega_nhat %%%
sigma = sqrt(Sigma_nhat(locd,:)); % standard deviations of moments
Omega_nhat = Sigma_nhat ./ (repmat(sigma,J,1).*kron(sigma,ones(J,1)));
Omega_nhat(Omega_nhat<-1)= -1; 
Omega_nhat(Omega_nhat>1) =  1;
Omega_nhat(isnan(Omega_nhat)) = 0;
end

function Tn_phat=get_submoment_AB(dat,As,nu_cA,m_nbar,moment_selectmatrix)
%%% step 1: calculate bootstrap moments
J=length(As);
n=size(dat,1);
S=size(dat,2);
% warning('off','MATLAB:singularMatrix')
% warning('off','MATLAB:nearlySingularMatrix')

Tn_phat=zeros(S,1); %%% store all S simulation test result

Qn=get_frequencies(dat,[0,1]);
P_Aboot=get_freq_events(As,Qn);

m_nbar_boot=nu_cA-P_Aboot;
m_n_phat=sqrt(n)*(m_nbar_boot-m_nbar).*(moment_selectmatrix); %%% Notice here we didnot delete zero elements yet

[sigma, Sigma_nhat, Omega_nhat, ~] = get_correl_AB(P_Aboot,As);
epsilon=0.012; % this value is based on the AB EMA(2012)

%%% step 3: calculate test-statistic for each simulation
for s=1:S
    omega = reshape(Omega_nhat(:,s),[J,J]);
    correct_comp=[epsilon-det(omega),0];    % based on p.2808 in AB
    sigma_ntilde=reshape(Sigma_nhat(:,s),[J,J])+max(correct_comp)*diag(sigma(:,s).^2);
    
    %%% find nonzero indexs and delete corresponding elements from variance
    %%% covariance matrix
    m_nr_star=m_n_phat(:,s);
    nonzero_index=find(m_nr_star);
    sigma_ntilde_phat=sigma_ntilde([nonzero_index],[nonzero_index]);
    m_nr_phat=m_nr_star([nonzero_index],1);
    
    %%% set objects used in QP %%%
    settings.verbose=0;
    params.Q=inv(sigma_ntilde_phat);
    params.m=m_nr_phat;
    
    %%% solve QPs using CVXGEN %%%
    if length(m_nr_phat) == 1
        [~,stat] = csolve1(params,settings);
        Tn_phat(s,1)=stat.optval;
    elseif length(m_nr_phat) == 2
        [~,stat] = csolve2(params,settings);
        Tn_phat(s,1)=stat.optval;
    elseif length(m_nr_phat) == 3
        [~,stat] = csolve3(params,settings);
        Tn_phat(s,1)=stat.optval;
    elseif length(m_nr_phat) == 4
        [~,stat] = csolve4(params,settings);
        Tn_phat(s,1)=stat.optval;
    elseif length(m_nr_phat) == 5
        [~,stat] = csolve5(params,settings);
        Tn_phat(s,1)=stat.optval;
    elseif length(m_nr_phat) == 6
        [~,stat] = csolve6(params,settings);
        Tn_phat(s,1)=stat.optval;
    end
end
end

%%% First revise is start from here, we need to put one more argument to 
function [kappa, eta1, eta2]=get_TP_SCF_AB(delta,J)
% Compute Tuning parameters and size correction factors
% delta   : S by 1 (S: # of simulations, each time simulation the off diagonal
%           smallest element
% Remark  : This file only use for alph=0.05, based on Table 1 in AB
%           EMA(2012)

%%% define bins in Table 1 %%%
S=length(delta);
intervals=[-0.95:0.05:0.95]; 
eta = 0.1;
edges=[-1-eta,-0.975,intervals,0.975,0.99,1+eta]; % -1.0001 instead of -1 is used to handle a numerical issue
[~,index]=histc(delta,edges);                  % store index that indicates the bin each delta belongs to

%%% values of kappa(delta) in Table 1 %%%
kappa_delta=[repmat(2.9,1,3),2.8,2.7,repmat(2.6,1,2),repmat(2.5,1,4),2.4,repmat(2.2,1,2)...
    repmat(2.1,1,3),repmat(2.0,1,2),1.9,1.8,1.5,repmat(1.4,1,2),repmat(1.3,1,2)...
    ,1.2,1.1,1.0,repmat(0.8,1,2),repmat(0.6,1,2),repmat(0.4,1,2),0.2,zeros(1,8)];

%%% values of eta1(delta) in Table 1 %%%
eta1_delta=[0.025,0.026,0.021,0.027,0.062,0.104,0.103,0.131,0.122,0.113,0.104,0.124,0.158,...
    0.133,0.138,0.111,0.082,0.083,0.074,0.082,0.075,0.114,0.112,0.083,0.089,0.058,0.055,...
    0.044,0.040,0.051,0.023,0.033,0.013,0.016,0.000,0.003,0.002,zeros(1,7)];
Eta2=[0.00,0.15,0.17,0.24,0.31,0.33,0.37,0.45,0.50]; %%% notice the first number stand for value of eta2 with 2 inequalities and so on
eta2_delta=Eta2(1,J-1);

%%% obtain kappa and size correction factor %%%
TP_SCF_delta=[kappa_delta',eta1_delta',repmat(eta2_delta,1,length(kappa_delta))']; 
TP_SCF_n=zeros(S,3); 

for s=1:S
    temp=index(s);
    TP_SCF_n(s,:)=TP_SCF_delta(temp,:);
end

kappa=TP_SCF_n(:,1);
eta1=TP_SCF_n(:,2);
eta2=TP_SCF_n(:,3);
end

function prop_pass = check_theta_in_CI_GH(dat,As,thetas,alph,B,hn,standopt)
% checks if theta is in GH's confidence interval
% dat: n by S (n: sample size, S: # of simulations)
% As: cell array of events - each event is a vector
% thetas: value of parameter to check (each theta is a row), n_thetas by 2
% alph: 1-alpha is the confidence level
% B: # of bootstrap replications
% hn: tuning parameter
% standopt: standardization option, if standopt=1 the test standardized.
%           Otherwise, standopt=0, which means the classic GH test without
%           standardization
% prop_pass: proportion of simulations where theta is contained in the confidence set, n_thetas by 1

S = size(dat,2);
J = length(As);
n_thetas = size(thetas,1);
prop_pass = NaN(n_thetas,1);

nu_cA0 = zeros(J,S);  % temporary
[~, bt_temp, c_ub]  = get_critical_value_GH(dat,As,nu_cA0,alph,B,hn,standopt);  % bt_temp and c_ub do not depend on theta (and nu_cA)
% c_ub: upper bound of the critical value
parfor thi=1:n_thetas
    theta = thetas(thi,:);
    nu_cA = zeros(J,S);  % theoretical max prob
    for j=1:J
        nu_cA(j,:) = get_nu_conj(theta,As{j});
    end

    Tn = get_statistic_GH(dat,As,nu_cA,standopt);  % statistic
    if all(Tn>c_ub)
        prop_pass(thi) = 0;
    else
        c = get_critical_value_GH(dat,As,nu_cA,alph,B,hn,standopt,bt_temp);  % critical value
%         prop_pass(thi) = mean(Tn<=c);
        prop_pass(thi) = 1- mean(Tn>c);
    end
end
end

function Tn = get_statistic_GH(dat,As,nu_cA,standopt)
% computes GH's statistic
% dat  : n by S (n: sample size, S: # of simulations)
% As   : cell array of events - each event is a vector
% nu_cA: upper probability
% standOpt: standardization option, if standopt=1 the test standardized.
%           Otherwise, standopt=0, which means the classic GH test without
%           standardization
J = length(As);
n = size(dat,1);
S = size(dat,2);

Q = get_frequencies(dat,[0,1]);  % empirical frequencies of outcomes
P_A = get_freq_events(As,Q);  % empirical frequencies of events

% statistic

% This is for standardization denominator choose
if standopt==1;
    shat_A = sqrt(P_A .* (1-P_A));
elseif standopt==0;
    shat_A=ones(J,S);
end

Tn1 = bsxfun(@rdivide, P_A-nu_cA , shat_A);
Tn1(isnan(Tn1)) = 0;
Tn = sqrt(n)*max(Tn1);
end

function [c, bt_temp, c_ub] = get_critical_value_GH(dat,As,nu_cA,alph,B,hn,standopt,bt_temp)
% computes critical value of GH
% dat  : n by S (n: sample size, S: # of simulations)
% As   : cell array of events - each event is a vector
% nu_cA: upper probability
% alph : 1-alpha is the confidence level
% B    : # of bootstrap replications
% hn   : tuning parameter for GMS
% standOpt: standardization option, if standopt=1 the test standardized.
%           Otherwise, standopt=0, which means the classic GH test without
%           standardization
% c: vector of critical values (for each simulated data set)
% bt_temp: component that does not depend on theta
% c_ub: upper bound of c that does not depend on theta. (scalar. max over data sets)

J = length(As);
n = size(dat,1);
S = size(dat,2);

% Get empirical frequencies
Q = get_frequencies(dat,[0,1]);  % empirical frequencies of outcomes
P_A = get_freq_events(As,Q);
shat_A = sqrt(P_A .* (1-P_A));

% GMS
if standopt==1;
    Chn = sqrt(n)*(P_A-nu_cA)./shat_A >= -hn;
elseif standopt==0;
    Chn=sqrt(n)*(P_A-nu_cA)>=-hn;
end
% Chn_big = matbig(Chn,B);
Chn_big = reshape(repmat(Chn,B,1),J,B,S);  % faster

% bootstrap empirical freqs
% Asub = [11,10,01];
% Jsub = size(Asub,2);
if exist('bt_temp','var')==0  % if bt_temp is not passed to this function, compute it
    rng(12345)
    bt_temp = bootstrp(B,@(x)get_standized_mm(x,As,P_A,standopt),dat);
    bt_temp = reshape(bt_temp,B,J,S);
    bt_temp = permute(bt_temp,[2,1,3]);
    Tb_temp = squeeze(max(bt_temp,[],1));
    c_ub  = max(quantile(Tb_temp,1-alph,1));  % upper bound of c
end
% Tb = squeeze(max(bt_temp.*Chn_big,[],1));  % theoretically inaccurate, but it works
% % The following lines compute logically correct Tb. For large B, the above should be the same as below
bt_temp2 = bt_temp;
bt_temp2(~Chn_big) = 0;
Tb = squeeze(max(bt_temp2,[],1));
c  = quantile(Tb,1-alph,1);
end

function dat = generate_data(theta,S,n,eqsel)
% generates data
% theta: parameters
% S: Number of Monte Carlo simulations
% n: sample size for each simulation
% eqsel: equilibrium selection
% dat: generated data, n by S
%%% Monte Carlo Simulation for the testing

rng('default');
rng(123456);

switch eqsel.proc
    case 'IID'  % IID Data Generation %%%
        u1=rand([n,S]); % Total Data Set for u1
        vi=rand(n,S)<=eqsel.p; %%% Bernoulli random variable
        G_result=zeros(n,S);  %%% Correspondence
        G_result( (u1<theta(1))) = 1;
        G_result( (u1>theta(2))) = 0;
        G_result( (u1>=theta(1)) & (u1<=theta(2)) & vi ) = 1;
        dat = G_result;
        
    case 'NonIID'
        get_hi = @(i) 2.^(2.^ceil(log(log(i)./log(2))./log(2)));
        nL = get_hi(n);
        u1L=rand([nL,S]); % Total Data Set for u1
        hi_previous = 1;
        datL = zeros(nL,S);
        while 1
            hi_current = get_hi(hi_previous+1);
            if hi_current==hi_previous
                error('K error')
            end
            cluster_current = (hi_previous+1):hi_current;
            G1cum = sum(u1L(1:hi_current,:)<theta(1));
            G0cum = sum(u1L(1:hi_current,:)>theta(2));
            vii = (G1cum./(G1cum+G0cum)) > (theta(1)/(theta(1)+1-theta(2)));
            vi = repmat(vii,length(cluster_current),1);
            G1 = u1L(cluster_current,:)<theta(1);
            G0 = u1L(cluster_current,:)>theta(2);

            dat_cluster = zeros(size(G1));
            dat_cluster(G1) = 1;
            dat_cluster((~G1) & (~G0) & vi) = 1;
            datL(cluster_current,:) = dat_cluster;
            hi_previous = hi_current;
            if hi_current>=n; break; end
        end
        dat = datL(1:n,:);
end

end

function prop_pass = check_theta_in_CI_EKS(dat,As,thetas,alph)
% checks if theta is in the confidence interval
% dat: n by S (n: sample size, S: # of simulations)
% As: cell array of events - each event is a vector
% thetas: value of parameter to check (each theta is a row), n_thetas by 2
% alph: 1-alpha is the confidence level
% prop_pass: proportion of simulations where theta is contained in the confidence set, n_thetas by 1

S = size(dat,2);
J = length(As);
n_thetas = size(thetas,1);
prop_pass = NaN(n_thetas,1);
GHK_draws = init_GHK(J);
parfor thi=1:n_thetas
    theta = thetas(thi,:);
    nu_A = zeros(J,S);  % theoretical min prob (or belief function)
    for j=1:J
        nu_A(j,:) = get_nu(theta,As{j});
    end

    var_mat = zeros(J,J);  % variance matrix
    for j=1:J
        for jp=1:j
            B = intersect(As{j},As{jp},'stable');
            var_mat(j,jp) = get_nu(theta,B) - get_nu(theta,As{j})*get_nu(theta,As{jp});
            var_mat(jp,j) = var_mat(j,jp);
        end
    end
    sd_A = sqrt(diag(var_mat));  % sd of nu

    Tn = get_statistic_EKS(dat,As,nu_A,sd_A);  % statistic
    if all(Tn>sqrt(sum(sd_A>0)/alph))  % c is at least sqrt(sum(sd_A>0)/alph)
        prop_pass(thi) = 0;
    else
        c = get_critical_value_EKS(var_mat,sd_A,alph,GHK_draws);  % critical value
        prop_pass(thi) = mean(Tn<=c);
    end
end
end

function Tn = get_statistic_EKS(dat,As,nu,sd_A)
% computes the new statistic
% dat: n by S (n: sample size, S: # of simulations)
% As: cell array of events - each event is a vector
% nu_A: belief fucntion (lower prob)
% sd_A: sd of nu

n = size(dat,1);

Q = get_frequencies(dat,[0,1]);  % empirical frequencies of outcomes
P_A = get_freq_events(As,Q);

% statistic
Tn1 = bsxfun(@rdivide, nu - P_A, sd_A);
Tn1(isnan(Tn1)) = 0;
Tn = sqrt(n)*max(Tn1);
end

function c = get_critical_value_EKS(var_mat,sd_A,alph,GHK_draws)
% computes critical value
% var_mat: covariance matrix
% sd_A: sd vector
% alph: 1-alpha is the confidence level
% GHK_draws: random sequence for GHK

var_mat1 = var_mat(sd_A>0,sd_A>0);
if isempty(var_mat1)  % if var_mat=0
    c = 0;
else
%     corr_mat1 = corrcov(var_mat1);  % equivalently,  corr_mat1=var_mat1./(sd_A1*sd_A1');
    sd_A1 = sd_A(sd_A>0);
    corr_mat1=var_mat1./(sd_A1*sd_A1');
    J1 = size(corr_mat1,1);
    c = fzero(@(c) GHK_cdf(corr_mat1,c*ones(J1,1),GHK_draws)-(1-alph), [0,sqrt(J1/alph)]); % critical value
end
end

function freq = get_frequencies(dat,outcomes)
% computes empirical frequencies
% dat: n by S (n: sample size, S: # of simulations)
% outcomes: a vector of outcomes to count (K by 1)
% freq: a matrix whose colume is empirical frequencies (K by S)

K = length(outcomes);
S = size(dat,2);
freq = zeros(K,S);  % empirical frequencies of outcomes
for k=1:K
    freq(k,:) = mean(dat == outcomes(k));
end
end

function P_A = get_freq_events(As,Q)
% compute frequencies of events from those for outcomes
% As : cell array of events - each event is a vector
% Q  : Jsub x M matrix of frequencies of outcomes
% P_A: J x M matrix of frequencies of events

M = size(Q,2);
J = length(As);
P_A=zeros(J,M);  % empirical frequencies of events

for j=1:J
    Aj = As{j};
    if ~isempty(find(Aj == 0,1)); P_A(j,:) = P_A(j,:) + Q(1,:);end
    if ~isempty(find(Aj == 1,1)); P_A(j,:) = P_A(j,:) + Q(2,:);end
end
end

function pr = GHK_cdf( varmat,Ubound,GHK_draws)
% computes cdf of multivariate normal
% varmat : variance matrix (positive semi-definite)
% UBound: upper bound, a column vector
% Nsim: number of simulations (it seems at least 10^5 is required)
% GHK_draws: random sequence for GHK
% pr: probability of cdf, computed by GHK simulator

J = size(Ubound,1);
Nsim = size(GHK_draws,2);

[L1,D1] = ldl(varmat);
L = L1*(sqrt(abs(D1)));  % varmat is decomposed into L*L' where L is a lower triangular matrix
% L(k,k) may be 0
% 1/0, and -1/0 is ok, because normcdf(+-inf) is handled
% 0/0 is problematic but it rarely happens

a = zeros(J,Nsim);
a(1,:) = normcdf( Ubound(1)/L(1,1) );
z = zeros(J, Nsim);
for i = 2:J
  z(i-1,:) = norminv(GHK_draws(i-1,:).*a(i-1,:));
  E = L(i,1:(i-1))*z(1:(i-1),:);
  b = (Ubound(i)-E)/abs(L(i,i));    % reason for abs(): diagonal of L is non-negative, but it may be negative due to numerical errors
  b(isnan(b))=inf;  % when Ubound(i)-E=0 and L(i,i)=0, assume the realized samples "in the area"
  a(i,:) = normcdf(b);
end
pr = mean(prod(a,1));
end

function GHK_draws = init_GHK(dim)
% % halton sequence: according to the stata manual on ghk(), halton is good for dim<10
halton = haltonset(dim);
N = 10^4+1;  % 10^4 seems good enough for dim=6
GHK_draws = halton(2:N,:)';

% % random
% rng(12345)
% N_sim = 10^5;  % 10^5 or 10^6 seems good for J=6
% GHK_draws = rand(dim,N_sim);
end

function nu = get_nu(theta,A)
% computes the theoretical lower bound or belief function
% theta: parameters, (each theta is a row), n_theta by 2
% A: a COLUMN vector representing the event to evaluate
%    assumed to be SORTED in descending order
%    if not, run A=sort(A,'descend');
% nu: vector of belief function values (n_theta by 1)

if isempty(A)
    nu = 0;
elseif A == 0
    nu = 1-theta(:,2);
elseif A == 1
    nu = theta(:,1);
else
    disp('error: wrong event?')
end
end

function nu_conj = get_nu_conj(theta,A)
% computes the theoretical upper bound or belief function
% theta: parameter
% A: a COLUMN vector representing the event to evaluate
%    assumed to be SORTED in descending order
%    if not, run A=sort(A,'descend');
% nu_conj: value of belief function

A_complement = setdiff([0; 1],A,'stable');
nu_conj = 1 - get_nu(theta,A_complement);
end

function Tb=get_standized_mm(dat, As,P_A, standopt)
    %%% First we need to notice we are going to pass this function into bootstrp function
    %%% The dat in this function stand for the bootstrap data from original
    %%% sample 
    J=length(As);
    n=size(dat,1);
    S=size(dat,2);
    
    Qn=get_frequencies(dat,[0,1]);
    P_Aboot=get_freq_events(As,Qn);
    
    if standopt==1
        Sn=sqrt(P_Aboot.*(1-P_Aboot));
    elseif standopt==0;
        Sn=ones(J,S);
    end
    
    Tb=sqrt(n)*((P_Aboot-P_A)./(Sn));
end
